@@ -1,4 +1,6 @@
 #' @importFrom cli cat_bullet cat_line cat_rule rule symbol
+#' @importFrom checkmate assert_count assert_names assert_integerish
+#' @importFrom checkmate test_integerish assert_integer assert_character
 #' @importFrom data.table copy data.table dcast rbindlist
 #' @importFrom data.table setattr setcolorder setnames setorder setorderv
 #' @importFrom stringi stri_locate_first_regex stri_replace_first_regex "stri_sub<-"
@@ -11,10 +13,10 @@ NULL
 get_idd_group_index <- function (idd_env, group = NULL) {
     if (is.null(group)) return(idd_env$group$group_id)
 
-    assert(are_string(group))
+    assert_character(group, any.missing = FALSE)
 
     res <- idd_env$group[J(group), on = "group_name", group_id]
-    if (anyNA(res)) abort_bad_key("error_group_name", "group name", group)
+    if (anyNA(res)) abort_bad_key("group name", group)
     res
 }
 # }}}
@@ -22,61 +24,63 @@ get_idd_group_index <- function (idd_env, group = NULL) {
 get_idd_group_name <- function (idd_env, group = NULL) {
     if (is.null(group)) return(idd_env$group$group_name)
 
-    assert(are_count(group))
+    assert_integerish(group, lower = 1L, any.missing = FALSE)
 
     res <- idd_env$group[J(group), on = "group_id", group_name]
-    if (anyNA(res)) abort_bad_key("error_group_id", "group index", group)
+    if (anyNA(res)) abort_bad_key("group index", group)
     res
 }
 # }}}
 
 # CLASS
 # get_idd_class {{{
-# Get class data
-# @param idd_env An environment or list contains IDD tables including class,
-#        field, and reference.
-# @param class An integer vector of valid class indexes or a character vector
-#        of valid class names. If `NULL`, all classes are returned.
-# @param property A character vector of column names in class table to return.
-#        If `NULL`, only class index columns are returned, plus column `rleid`.
-# @param underscore If `TRUE`, input class name will be converted into
-#        underscore style name first and column `class_name_us` will be used
-#        for matching.
-# @return A data.table containing specified columns.
+#' Get class data
+#'
+#' @param idd_env An environment or list contains IDD tables including class,
+#'        field, and reference.
+#' @param class An integer vector of valid class indexes or a character vector
+#'        of valid class names. If `NULL`, all classes are returned.
+#' @param property A character vector of column names in class table to return.
+#'        If `NULL`, only class index columns are returned, plus column `rleid`.
+#' @param underscore If `TRUE`, input class name will be converted into
+#'        underscore style name first and column `class_name_us` will be used
+#'        for matching.
+#'
+#' @return A data.table containing specified columns.
+#' @keywords internal
+#' @export
 get_idd_class <- function (idd_env, class = NULL, property = NULL, underscore = FALSE) {
+    cols <- setdiff(CLASS_COLS$index, "class_name_us")
+
     if (is.null(class)) {
-        cols <- setdiff(CLASS_COLS$index, "class_name_us")
-        if (is.null(property)) {
-            return(idd_env$class[, .SD, .SDcols = cols])
+        # very odd way to subset columns but is way faster that others
+        # ref: https://github.com/Rdatatable/data.table/issues/3477
+        if (is.null(property)) return(fast_subset(idd_env$class, cols))
+
+        if ("group_name" %chin% property) {
+            property <- setdiff(property, "group_name")
+            add_group <- TRUE
         } else {
-            if ("group_name" %chin% property) {
-                property <- setdiff(property, "group_name")
-                add_group <- TRUE
-            } else {
-                add_group <- FALSE
-            }
-
-            res <- idd_env$class[, .SD, .SDcols = unique(c(cols, property))]
-
-            if (add_group) {
-                add_joined_cols(idd_env$group, res, "group_id", "group_name")
-            }
-
-            return(res)
+            add_group <- FALSE
         }
+
+        # very odd way to subset columns but is way faster
+        # ref: https://github.com/Rdatatable/data.table/issues/3477
+        res <- fast_subset(idd_env$class, unique(c(cols, property)))
+
+        if (add_group) add_joined_cols(idd_env$group, res, "group_id", "group_name")
+
+        return(res)
     }
 
     cls_in <- recognize_input(class, "class", underscore)
     res <- join_from_input(idd_env$class, cls_in, "group_id")
-    set(res, NULL, "class_name_us", NULL)
 
-    property <- property %||% ""
-    if ("group_name" %chin% property) {
+    if (!is.null(property) && "group_name" %chin% property) {
         add_joined_cols(idd_env$group, res, "group_id", "group_name")
     }
-    clean_class_property(res, property)
 
-    res
+    fast_subset(res, c("rleid", unique(c(cols, property))))
 }
 # }}}
 # get_idd_class_field_num {{{
@@ -98,14 +102,15 @@ get_idd_class <- function (idd_env, class = NULL, property = NULL, underscore =
 #       - The acceptable field number will be the field index of the last
 #         field in the last extensible group.
 get_idd_class_field_num <- function (dt_class, num = NULL) {
-    if (!is.null(num)) assert(are_count(num))
+    assert_integer(num, lower = 1L, any.missing = FALSE, null.ok = TRUE)
 
     dt_class <- add_rleid(dt_class, "class")
 
     # directly return num of fields in class
-    assert(has_name(dt_class, c("num_fields", "min_fields", "last_required", "num_extensible", "first_extensible")))
+    assert_names(names(dt_class), must.include = c("num_fields", "min_fields", "last_required", "num_extensible", "first_extensible"))
 
     if (!nrow(dt_class)) {
+        set(dt_class, NULL, "class_rleid", NULL)
         set(dt_class, NULL, "input_num", integer(0L))
         set(dt_class, NULL, "acceptable_num", integer(0L))
         return(dt_class)
@@ -114,13 +119,13 @@ get_idd_class_field_num <- function (dt_class, num = NULL) {
     if (is.null(num)) {
         set(dt_class, NULL, "input_num", 0L)
     } else {
-        assert(have_same_len(dt_class, num))
+        assert_same_len(dt_class, num)
         set(dt_class, NULL, "input_num", as.integer(num))
     }
 
     # get index of the last field in the last extensible group
     set(dt_class, NULL, "last_extensible", 0)
-    if (nrow(dt_class[num_extensible > 0L])) {
+    if (any(dt_class$num_extensible > 0L)) {
         dt_class[
             num_extensible > 0L & input_num > first_extensible,
             last_extensible :=
@@ -144,16 +149,44 @@ get_idd_class_field_num <- function (dt_class, num = NULL) {
     dt_class
 }
 # }}}
-# clean_class_property {{{
-clean_class_property <- function (dt, property) {
-    col_del <- setdiff(CLASS_COLS$property, property)
-    if (length(col_del)) set(dt, NULL, col_del, NULL)
-    dt
+# get_idd_class_unique {{{
+get_idd_class_unique <- function (idd_env) {
+    idd_env$class[J(TRUE), on = "unique_object", nomatch = NULL]
+}
+# }}}
+# get_class_component_name {{{
+get_class_component_name <- function (class) {
+    nm <- stri_extract_first_regex(class, "^.+?(?=:)")
+    nm[is.na(nm)] <- class[is.na(nm)]
+    nm
 }
 # }}}
 
 # FIELD
 # get_idd_field {{{
+#' Get field data
+#'
+#' @param idd_env An environment or list contains IDD tables including class,
+#'        field, and reference.
+#' @param class An integer vector of valid class indexes or a character vector
+#'        of valid class names.
+#' @param field An integer vector of valid field indexes or a character
+#'        vector of valid field names (can be in in underscore style).  `class`
+#'        and `field` should have the same length.
+#' @param property A character vector of column names in field table to return.
+#' @param underscore If `TRUE`, input class name and field names will be
+#'        converted into underscore style name first and column `class_name_us`
+#'        and `field_name_us` will be used for matching.
+#' @param no_ext If `TRUE`, no new extensible groups will be added even if there
+#'        are no matched input found and an error will be issued right away.
+#' @param complete If `TRUE`, at least fields till the current whole extensible
+#'        group will be returned. A new column named "matched_rleid" will be
+#'        created (when `property` is NULL) indicating if given field has been
+#'        matched or not.
+#'
+#' @return A data.table containing specified columns.
+#' @keywords internal
+#' @export
 get_idd_field <- function (idd_env, class, field = NULL, property = NULL, all = FALSE,
                            underscore = TRUE, no_ext = FALSE, complete = FALSE) {
     if (is.null(field)) {
@@ -161,8 +194,12 @@ get_idd_field <- function (idd_env, class, field = NULL, property = NULL, all =
     } else {
         res <- get_idd_field_from_which(idd_env, class, field, underscore, no_ext, complete, all)
     }
-    if (has_name(res, "field_name_us")) set(res, NULL, "field_name_us", NULL)
-    clean_field_property(res, property %||% "")
+
+    cols <- c("rleid", "class_id", "class_name", "field_id", "field_index", "field_name", "field_in")
+    if (length(col_del <- setdiff(names(res), c(cols, property)))) {
+        set(res, NULL, col_del, NULL)
+    }
+
     res
 }
 # }}}
@@ -185,32 +222,9 @@ get_idd_field_in_class <- function (idd_env, class, all = FALSE, underscore = TR
 }
 # }}}
 # get_idd_field_from_which {{{
-# Get specified field data
-# @param idd_env An environment or list contains IDD tables including class,
-#        field, and reference.
-# @param class An integer vector of valid class indexes or a character vector
-#        of valid class names or a data.table that contains column `class_id`
-#        and `rleid`. If a data.table that contains a column `object_id`, that
-#        column will be preserved.
-# @param field An integer vector of valid field indexes or a character
-#        vector of valid field names (can be in in underscore style).  `class`
-#        and `field` should have the same length.
-# @param property A character vector of column names in field table to return. If
-#        `NULL`, all columns from IDD field table will be returned, plus column
-#        `rleid`, `object_id` (if applicable) and `matched_rleid` (if
-#        `complete` is `TRUE`).
-# @param underscore If `TRUE`, input class name and field names will be
-#        converted into underscore style name first and column `class_name_us`
-#        and `field_name_us` will be used for matching.
-# @param no_ext If `TRUE`, no new extensible groups will be added even if there
-#        are no matched input found and an error will be issued right away.
-# @param complete If `TRUE`, at least fields till the current whole extensible
-#        group will be returned. A new column named "matched_rleid" will be
-#        created (when `property` is NULL) indicating if given field has been
-#        matched or not.
 get_idd_field_from_which <- function (idd_env, class, field, underscore = TRUE,
                                       no_ext = FALSE, complete = FALSE, all = FALSE) {
-    assert_valid_type(field, "field")
+    assert_valid_type(field, "Field Index|Name")
 
     # class properties used for min required field num calculation
     col_prop <- c("num_fields", "min_fields", "last_required", "num_extensible",
@@ -239,9 +253,9 @@ get_idd_field_from_which <- function (idd_env, class, field, underscore = TRUE,
     }
     # }}}
 
-    assert(have_same_len(dt_in, field), prefix = "class and field")
+    assert_same_len(dt_in, field, "class and field")
 
-    if (all(are_count(field))) {
+    if (test_integerish(field, lower = 1L, any.missing = FALSE)) {
         # from field index {{{
         field <- as.integer(field)
         set(dt_in, NULL, c("field_index", "field_in"), list(field, field))
@@ -255,7 +269,7 @@ get_idd_field_from_which <- function (idd_env, class, field, underscore = TRUE,
         # check invalid field index
         if (dt_in[field_in > acceptable_num, .N > 0L]) {
             invld_idx <- dt_in[field_in > acceptable_num]
-            abort_bad_field("error_bad_field_index", "index", invld_idx)
+            abort_bad_field("index", invld_idx)
         }
 
         # handle extensible fields
@@ -266,7 +280,7 @@ get_idd_field_from_which <- function (idd_env, class, field, underscore = TRUE,
 
         # stop if adding new extensible groups is not allowed
         if (no_ext && nrow(dt_in[num > 0L])) {
-            abort_bad_field("error_bad_field_index", "index", dt_in[num > 0L])
+            abort_bad_field("index", dt_in[num > 0L])
         }
 
         # add extensible groups
@@ -348,12 +362,12 @@ get_idd_field_from_which <- function (idd_env, class, field, underscore = TRUE,
             # invalid field names for non-extensible classes
             if (any(dt_nom$class_id %in% idd_env$class[J(0L), on = "num_extensible", class_id])) {
                 invld_non_ext <- dt_nom[class_id %in% idd_env$class[J(0L), on = "num_extensible", class_id]]
-                abort_bad_field("error_bad_field_name", "name", clean_errnm_dt(invld_non_ext))
+                abort_bad_field("name", clean_errnm_dt(invld_non_ext))
             }
 
             # if all names not found are in extensible class
             if (no_ext) {
-                abort_bad_field("error_bad_field_name", "name", clean_errnm_dt(dt_nom))
+                abort_bad_field("name", clean_errnm_dt(dt_nom))
             }
 
             # get number of field names to check per class
@@ -383,7 +397,7 @@ get_idd_field_from_which <- function (idd_env, class, field, underscore = TRUE,
                     num_extensible_group = num_extensible_group + num
                     )]
                 idd_env <- del_idd_extensible_group(idd_env, dt_ext)
-                abort_bad_field("error_bad_field_name", "name",
+                abort_bad_field("name",
                     add_class_property(idd_env, invld_nm, c("min_fields", "num_fields")),
                     "\n\nNOTE: For extensible fields, new one will be added only ",
                     "when all previous extensible groups exist."
@@ -396,6 +410,7 @@ get_idd_field_from_which <- function (idd_env, class, field, underscore = TRUE,
                 set(dt_ext_join, NULL, col_prop, NULL)
                 set(dt_join, NULL, col_prop, NULL)
                 fld <- append_dt(dt_join[!is.na(field_id)], dt_ext_join)
+                setorderv(fld, "field_rleid")
                 set(fld, NULL, "field_rleid", NULL)
                 # }}}
             } else {
@@ -420,114 +435,309 @@ get_idd_field_from_which <- function (idd_env, class, field, underscore = TRUE,
         # }}}
     }
 
-    set(fld, NULL, "field_name_us", NULL)
     fld
 }
 # }}}
-# clean_field_property {{{
-clean_field_property <- function (dt, property) {
-    col_del <- setdiff(FIELD_COLS$property, property)
-    if (length(col_del)) set(dt, NULL, col_del, NULL)
-    dt
-}
-# }}}
 
 # REFERENCES
-# get_recref {{{
-get_recref <- function (reference, id, col_on, col_rec, dep, max = Inf) {
-    drill <- if (length(col_on) == 2L) TRUE else FALSE
-    ref <- reference[0L]
-    set(ref, NULL, "dep", integer())
-    dep <- dep
-    col <- col
-
-    get_ref <- function (id) {
-        if (drill) {
-            cur_ref <- reference[J(id), on = col_on[[1L]], nomatch = 0L]
-            drill <<- FALSE
-            col_on <<- col_on[[2L]]
+# get_recursive_relation {{{
+get_recursive_relation <- function (all_ref, init_ref, init_dep, max_dep,
+                                    col_ref, col_rev, include = NULL,
+                                    both = FALSE, match_all = FALSE) {
+    if (both) {
+        if (substring(col_ref, 1, 3) == "src") {
+            col_fld <- "src_field_id"
+            col_val <- "src_value_id"
         } else {
-            cur_ref <- reference[J(id), on = col_on, nomatch = 0L]
+            col_fld <- "field_id"
+            col_val <- "value_id"
         }
+        if (!col_val %chin% names(init_ref)) col_val <- col_fld
 
-        set(cur_ref, NULL, "dep", dep)
-        if (!nrow(cur_ref)) return(ref)
-        ref <<- rbindlist(list(ref, cur_ref))
-        if (dep == max) return(ref)
-        dep <<- dep + 1L
-        get_ref(cur_ref[[col_rec]])
+        # this assume that one class-name-reference is always followed by one
+        # field value reference
+        redundant <- init_ref[, by = c(col_rev, col_fld), {
+            # for field-reference only
+            if (!IDDFIELD_SOURCE$class %in% src_enum) {
+                id <- integer()
+            } else {
+                id <- setdiff(
+                    get(col_val)[src_enum == IDDFIELD_SOURCE$class],
+                    get(col_val)[src_enum == IDDFIELD_SOURCE$field]
+                )
+            }
+            list(id)
+        }]
+        setnames(redundant, c(col_rev, col_fld, col_val))
+        set(redundant, NULL, "src_enum", IDDFIELD_SOURCE$class)
+
+        if (nrow(redundant)) init_ref <- init_ref[!redundant, on = c(col_rev, col_val, "src_enum")]
     }
 
-    get_ref(id)
+    # parent class or object
+    parent <- init_ref[[col_rev]]
+
+    # store classes or objects needed to be removed later
+    del <- list()
+
+    # log depth
+    dep <- init_dep
+
+    ref <- init_ref
+    cur_ref <- init_ref
+    while (dep < max_dep && nrow(cur_ref)) {
+        # skip if specified classes/objects are matched
+        if (!match_all && !is.null(include)) {
+            skip <- unique(cur_ref[J(include), on = col_ref, col_rev, with = FALSE][[1L]])
+            if (length(skip)) {
+                cur_ref <- cur_ref[!J(skip), on = col_rev]
+
+                # should also remove all unmatched in the init match
+                if (dep == 0L) {
+                    exclu <- ref[J(unique(skip)), on = col_rev][!J(include), on = col_ref]
+                    ref <- ref[!exclu, on = c(col_rev, col_ref)]
+                }
+            }
+        }
+        # if all are matched, stop
+        if (!nrow(cur_ref)) break
+
+        # get current indirectly ref
+        new_ref <- all_ref[J(unique(cur_ref[[col_ref]])), on = col_rev, nomatch = 0L]
+
+        if (both) {
+            # this assume that one class-name-reference is always followed by one
+            # field value reference
+            redundant <- new_ref[, by = c(col_rev, col_fld), {
+                # for field-reference only
+                if (!IDDFIELD_SOURCE$class %in% src_enum) {
+                    id <- integer()
+                } else {
+                    id <- setdiff(
+                        get(col_val)[src_enum == IDDFIELD_SOURCE$class],
+                        get(col_val)[src_enum == IDDFIELD_SOURCE$field]
+                    )
+                }
+                list(id)
+            }]
+            setnames(redundant, c(col_rev, col_fld, col_val))
+            set(redundant, NULL, "src_enum", IDDFIELD_SOURCE$class)
+
+            if (nrow(redundant)) new_ref <- new_ref[!redundant, on = c(col_rev, col_val, "src_enum")]
+        }
+
+        # get classes that do not going any deeper
+        # those classes should be removed
+        if (!is.null(include)) {
+            del <- c(del,
+                list(setattr(setdiff(cur_ref[[col_ref]], c(include, new_ref[[col_rev]])), "dep", dep))
+            )
+        }
+
+        cur_ref <- new_ref
+
+        # add depth
+        dep <- dep + 1L
+        # set depth value
+        set(cur_ref, NULL, "dep", dep)
+        # merge into the main results
+        ref <- rbindlist(list(ref, cur_ref))
+
+        # remove self reference
+        cur_ref <- cur_ref[!J(parent), on = col_ref]
+    }
+
+    # should search backwards to only include paths related to specified
+    # classes/objects
+    if (!is.null(include) && nrow(ref)) ref <- del_recursive_relation(ref, del, include, col_ref, col_rev)
+
+    ref
 }
 # }}}
+# del_recursive_relation {{{
+del_recursive_relation <- function (ref, target, keep, col_ref, col_rev) {
+    # split ref by depth
+    ref <- split(ref, by = "dep")
+
+    # get class/object indices needed to be removed in former level
+    not_found <- unique(ref[[length(ref)]][!J(keep), on = col_ref, .SD, .SDcols = col_rev][[1L]])
+    # only keep specified classes for current level
+    ref[[length(ref)]] <- ref[[length(ref)]][J(keep), on = col_ref, nomatch = 0L]
+    # NOTE: should include classes that contain valid ref
+    not_found <- setdiff(not_found, ref[[length(ref)]][[col_rev]])
+
+    for (i in rev(seq_along(ref[-length(ref)]))) {
+        # include specified classes to be removed
+        not_found <- c(not_found, target[[i]])
+        # get class/object indices needed to be removed in former level
+        next_not_found <- unique(ref[[i]][J(not_found), on = col_ref, .SD, .SDcols = col_rev][[1L]])
+        # remove at current level
+        if (length(not_found)) ref[[i]] <- ref[[i]][!J(not_found), on = col_ref]
+        not_found <- next_not_found
+    }
+
+    rbindlist(ref)
+}
+# }}}
+# combine_input_and_relation {{{
+combine_input_and_relation <- function (input, ref, type, direction) {
+    if (type == "idd") {
+        col_main <- "class_id"
+        col_sub <- "field_id"
+    } else if (type == "idf") {
+        col_main <- "object_id"
+        col_sub <- "value_id"
+    }
+
+    if (direction == "ref_by") {
+        col_main <- paste0("src_", col_main)
+        col_sub <- paste0("src_", col_sub)
+        setnames(input, c(col_sub, col_main))
+    }
+
+    if (!nrow(ref) || max(ref$dep) == 0L) {
+        set(ref, NULL, col_main, NULL)
+        ref <- ref[input, on = col_sub]
+        set(ref, NULL, "dep", 0L)
+    } else {
+        ref0 <- ref[J(0L), on = "dep"]
+        set(ref0, NULL, col_main, NULL)
+        ref0 <- ref0[input, on = col_sub]
+        set(ref0, NULL, "dep", 0L)
+        ref <- append_dt(ref0, ref[!J(0L), on = "dep"])
+    }
+
+    ref
+}
+# }}}
+
 # get_idd_relation {{{
-get_idd_relation <- function (idd_env, class = NULL, field = NULL, max_depth = NULL,
-                              name = FALSE, direction = c("ref_to", "ref_by"), keep_all = FALSE) {
-    assert(is.null(max_depth) || is_count(max_depth, TRUE))
+#' Get field relation data
+#'
+#' @param idd_env An environment or list contains IDD tables including class,
+#'        field, and reference.
+#' @param class_id An integer vector of valid class indexes. Should be `NULL` if
+#'        `field_id` is given.
+#' @param field_id An integer vector of valid field id. Should be `NULL` if
+#'        `class_id` is given.
+#' @param direction The relation direction to extract. Should be one of
+#'        `"ref_to"` or `"ref_by"`.
+#' @param underscore If `TRUE`, input class name and field names will be
+#'        converted into underscore style name first and column `class_name_us`
+#'        and `field_name_us` will be used for matching.
+#' @param depth If > 0, the relation is searched recursively. If `NULL`,
+#'        all possible recursive relations are returned. Default: `0`.
+#' @param name If `TRUE`, additional formatting columns are added and an
+#'        `IddRelation` object is returned. Default: `FALSE`.
+#' @param class,group A character vector of group names used for searching
+#'        relations. Default: `NULL`.
+#' @param keep If `TRUE`, all inputs are returned regardless they have any
+#'        relations with other fields or not. If `FALSE`, only input that have
+#'        relations with other fields are returned. Default: `FALSE`.
+#' @param class_ref Specify how to handle class-name-references. There are 3
+#'        options in total, i.e. `"none"`, `"both"` and `"all"`, with `"both"`
+#'        being the default.
+#'     * `"none"`: just ignore class-name-references.
+#'     * `"both"`: only include class-name-references if this object
+#'       also reference field values of the same one. This is the default
+#'       option.
+#'     * `"all"`: include all class-name-references. This is the most aggressive
+#'       option.
+#' @param match_all If `TRUE`, relation search will continue even though one
+#'        relation has been found. If `FALSE`, searching is stopped whenever one
+#'        relation is found in specified classes/groups. Default: `FALSE`.
+#'
+#' @return A data.table.
+#'
+#' @keywords internal
+#' @export
+get_idd_relation <- function (idd_env, class_id = NULL, field_id = NULL,
+                              direction = c("ref_to", "ref_by"), depth = 0L, name = FALSE,
+                              class = NULL, group = NULL, keep_all = FALSE,
+                              class_ref = c("both", "none", "all"), match_all = FALSE) {
+    assert_count(depth, null.ok = TRUE)
+    if (is.null(depth)) depth <- Inf
     direction <- match.arg(direction)
+    class_ref <- match.arg(class_ref)
 
     # get class reference
-    if (is.null(field)) {
-        if (is.null(class)) {
+    if (is.null(field_id)) {
+        if (is.null(class_id)) {
             id <- idd_env$class$class_id
         } else {
-            id <- get_idd_class(idd_env, class)$class_id
+            id <- get_idd_class(idd_env, class_id)$class_id
         }
-        col_on <- c("class_id", "field_id")
+        col_on <- "class_id"
     } else {
         # if no class is given, assume field are valid field ids
-        if (is.null(class)) {
-            id <- field
+        if (is.null(class_id)) {
+            id <- field_id
         } else {
-            id <- get_idd_field(idd_env, class, field)$field_id
+            abort("Should not specify both class id and field id at the same time",
+                "idd_relation"
+            )
         }
         col_on <- "field_id"
     }
 
     if (keep_all) {
-        fld <- idd_env$field[J(id), on = col_on[[1L]], .SD, .SDcols = c("field_id", "class_id")]
+        # make sure all input IDs appear in the result
+        fld <- idd_env$field[J(id), on = col_on, .SD, .SDcols = c("field_id", "class_id")]
     }
 
+    if (direction == "ref_by") col_on <- paste0("src_", col_on)
+
+    all_ref <- idd_env$reference
+
+    if (class_ref == "none") {
+        both <- FALSE
+        all_ref <- all_ref[!J(IDDFIELD_SOURCE$class), on = "src_enum"]
+    } else if (class_ref == "all") {
+        both <- FALSE
+    } else if (class_ref == "both") {
+        both <- TRUE
+    }
+
+    # init depth
+    dep <- 0L
+
+    # get first directly ref
+    cur_ref <- all_ref[J(id), on = col_on, nomatch = 0L]
+    set(cur_ref, NULL, "dep", if (nrow(cur_ref)) dep else integer())
+
     if (direction == "ref_to") {
-        col_rec <- "src_field_id"
+        col_ref <- "src_class_id"
+        col_rev <- "class_id"
     } else if (direction == "ref_by") {
-        col_on <- paste0("src_", col_on)
-        col_rec <- "field_id"
+        col_ref <- "class_id"
+        col_rev <- "src_class_id"
     }
 
-    # TODO: for extensible groups, only use the first group
-    dep <- 0L
-    if (is.null(max_depth)) max_depth <- Inf
-
-    ref <- get_recref(idd_env$reference, id, col_on, col_rec, dep, max_depth)
-
-    if (keep_all) {
-        if (direction == "ref_to") {
-            set(ref, NULL, "class_id", NULL)
-            if (!nrow(ref) || max(ref$dep) == 0L) {
-                ref <- ref[fld, on = "field_id"]
-                set(ref, NULL, "dep", 0L)
-            } else {
-                ref0 <- ref[J(0L), on = "dep"]
-                ref0 <- ref0[fld, on = "field_id"]
-                set(ref0, NULL, "dep", 0L)
-                ref <- append_dt(ref0, ref[!J(0L), on = "dep"])
-            }
-        } else {
-            set(ref, NULL, "src_class_id", NULL)
-            setnames(fld, c("src_field_id", "src_class_id"))
-            if (!nrow(ref) || max(ref$dep) == 0L) {
-                ref <- ref[fld, on = "src_field_id"]
-                set(ref, NULL, "dep", 0L)
-            } else {
-                ref0 <- ref[J(0L), on = "dep"]
-                ref0 <- ref0[fld, on = "src_field_id"]
-                set(ref0, NULL, "dep", 0L)
-                ref <- append_dt(ref0, ref[!J(0L), on = "dep"])
-            }
-        }
+    # restrict searching reference ranges
+    # NOTE: should do this depending on depth value
+    # This makes it possible to find recursive relations, e.g. how is the
+    # AirLoopHVAC related to Schedule:Compact?
+    cls_id <- NULL
+    if (!is.null(group)) {
+        grp_id <- get_idd_group_index(idd_env, group)
+        cls_id <- idd_env$class[J(grp_id), on = "group_id"]$class_id
     }
+    if (!is.null(class)) {
+        cls_id <- c(cls_id, get_idd_class(idd_env, class)$class_id)
+    }
+    if (depth == 0L && !is.null(cls_id)) {
+        cur_ref <- cur_ref[J(cls_id), on = col_ref, nomatch = 0L]
+    }
+
+    # no matched found for specified classes or groups
+    if (!is.null(cls_id) && !length(cls_id)) all_ref <- all_ref[0L]
+
+    # get recursive relation
+    ref <- get_recursive_relation(all_ref, cur_ref, dep, depth, col_ref,
+        col_rev, cls_id, both = both, match_all = match_all)
+
+    # keep all input
+    if (keep_all) ref <- combine_input_and_relation(fld, ref, "idd", direction)
 
     if (!name) return(ref)
 
@@ -559,21 +769,39 @@ add_idd_relation_format_cols <- function (idd_env, ref) {
 }
 # }}}
 
+# PROPERTY COLUMNS
 # add_class_id {{{
 add_class_id <- function (idd_env, dt) {
-    if (has_name(dt, "class_id")) return(dt)
     add_joined_cols(idd_env$class, dt, "class_name", "class_id")
 }
 # }}}
 # add_class_name {{{
 add_class_name <- function (idd_env, dt) {
-    if (has_name(dt, "class_name")) return(dt)
     add_joined_cols(idd_env$class, dt, "class_id", "class_name")
 }
 # }}}
 # add_class_property {{{
 add_class_property <- function (idd_env, dt, property) {
+    add_group <- FALSE
+    if ("group_name" %in% property) {
+        if ("group_id" %in% property) {
+            add_id <- FALSE
+            property <- property[property != "group_name"]
+        } else {
+            property <- c(property[property != "group_name"], "group_id")
+            add_id <- TRUE
+        }
+        add_group <- TRUE
+    }
+
     add_joined_cols(idd_env$class, dt, "class_id", property)
+
+    if (add_group) {
+        add_joined_cols(idd_env$group, dt, "group_id", "group_name")
+        if (add_id) set(dt, NULL, "group_id", NULL)
+    }
+
+    dt
 }
 # }}}
 # add_field_property {{{
@@ -582,15 +810,35 @@ add_field_property <- function (idd_env, dt, property) {
 }
 # }}}
 
+# UNIT CONVERSION
 # field_default_to_unit {{{
-field_default_to_unit <- function (dt_field, from, to) {
+field_default_to_unit <- function (idd_env, dt_field, from, to) {
+    if (has_names(dt_field, "value_id")) {
+        value_id <- dt_field$value_id
+    } else {
+        value_id <- NULL
+    }
+    if (has_names(dt_field, "value_chr")) {
+        setnames(dt_field, c("value_chr", "value_num"), paste0(c("value_chr", "value_num"), "-backup"))
+    }
     set(dt_field, NULL, "value_id", seq_along(dt_field$field_id))
+
+    cols_add <- NULL
+    if (!has_names(dt_field, "default_chr")) cols_add <- "default_chr"
+    if (!has_names(dt_field, "default_num")) cols_add <- c(cols_add, "default_num")
+    if (!is.null(cols_add)) add_field_property(idd_env, dt_field, cols_add)
+
     setnames(dt_field, c("default_chr", "default_num"), c("value_chr", "value_num"))
 
-    dt_field <- convert_value_unit(dt_field, from, to)
+    dt_field <- convert_value_unit(idd_env, dt_field, from, to)
 
-    set(dt_field, NULL, "value_id", NULL)
+    set(dt_field, NULL, "value_id", value_id)
     setnames(dt_field, c("value_chr", "value_num"), c("default_chr", "default_num"))
+
+    if (has_names(dt_field, "value_chr-backup")) {
+        setnames(dt_field, paste0(c("value_chr", "value_num"), "-backup"), c("value_chr", "value_num"))
+    }
+
     dt_field
 }
 # }}}
@@ -602,11 +850,9 @@ add_idd_extensible_group <- function (idd_env, class, num = NULL, strict = FALSE
 
     # stop if non-extensible class found
     if (strict && nrow(dt_cls[num_extensible == 0L])) {
-        abort("error_nonextensible_class",
-            paste0("Non-extensible class found: ",
-                collapse(dt_cls[num_extensible == 0L, unique(class_name)])
-            )
-        )
+        abort(paste0("Non-extensible class found: ",
+            collapse(dt_cls[num_extensible == 0L, unique(class_name)])
+        ), "non_extensible_class")
     }
 
     ext <- dt_cls[num_extensible > 0L & num > 0L]
@@ -721,13 +967,12 @@ del_idd_extensible_group <- function (idd_env, class, num = NULL, strict = FALSE
 
     # stop if non-extensible class found
     if (strict && nrow(dt_cls[num_extensible == 0L])) {
-        stop("Non-extensible class found: ",
-            collapse(dt_cls[num_extensible == 0L, unique(class_name)]),
-            call. = FALSE
-        )
+        abort(paste0("Non-extensible class found: ",
+            collapse(dt_cls[num_extensible == 0L, unique(class_name)])
+        ), "non_extensible_class")
     }
 
-    ext <- dt_cls[num_extensible > 0L]
+    ext <- dt_cls[num_extensible > 0L & num > 0L]
 
     if (!nrow(ext)) return(idd_env)
 
@@ -747,7 +992,7 @@ del_idd_extensible_group <- function (idd_env, class, num = NULL, strict = FALSE
         less <- errormsg_info(less)
         mes <- less[, paste0(info, ": ", left_fields, " left with ", last_required, " required.")]
         mes <- paste0("Failed to delete extensible groups. Number of field(s) left less than required:\n", mes)
-        abort("error_del_extensible", mes, data = less)
+        abort(mes, data = less)
     }
 
     # get field id to delete
@@ -772,16 +1017,16 @@ del_idd_extensible_group <- function (idd_env, class, num = NULL, strict = FALSE
 get_input_class_data <- function (idd_env, class, num = NULL) {
     if (is.data.frame(class)) {
         dt_cls <- class
-        assert(has_name(dt_cls,
-            c("min_fields", "num_fields", "num_extensible", "last_required", "num_extensible_group")
-        ))
+        assert_names(names(dt_cls),
+            must.include = c("min_fields", "num_fields", "num_extensible", "last_required", "num_extensible_group")
+        )
 
         if (is.null(num)) {
-            assert(has_name(dt_cls, "num"))
+            assert_names(names(dt_cls), must.include = "num")
             set(dt_cls, NULL, "num", as.integer(dt_cls$num))
         } else {
-            assert(are_count(num))
-            set(dt_cls, NULL, "num", as.integer(num))
+            num <- assert_integerish(num, lower = 1L, any.missing = FALSE, coerce = TRUE)
+            set(dt_cls, NULL, "num", num)
         }
 
     } else {
@@ -793,9 +1038,9 @@ get_input_class_data <- function (idd_env, class, num = NULL) {
             )
         )
 
-        assert(are_count(num))
-        assert(have_same_len(class, num))
-        set(dt_cls, NULL, "num", as.integer(num))
+        num <- assert_integerish(num, lower = 1L, any.missing = FALSE, coerce = TRUE, len = 1L)
+        assert_same_len(class, num)
+        set(dt_cls, NULL, "num", num)
     }
 }
 # }}}
@@ -803,7 +1048,7 @@ get_input_class_data <- function (idd_env, class, num = NULL) {
 # TABLE
 # get_idd_table {{{
 get_idd_table <- function (idd_env, class, all = FALSE) {
-    assert_valid_type(class, "class")
+    assert_valid_type(class, "Class Index|Name")
     fld <- get_idd_field(idd_env, class, all = all)[
         , .SD, .SDcols = c("class_name", "field_index", "field_name")
     ]
@@ -817,8 +1062,8 @@ get_idd_table <- function (idd_env, class, all = FALSE) {
 # STRING
 # get_idd_string {{{
 get_idd_string <- function (idd_env, class, leading = 4L, sep_at = 29L, sep_each = 0L, all = FALSE) {
-    assert_valid_type(class, "class")
-    assert(is_count(sep_each, TRUE))
+    assert_valid_type(class, "Class Index|Name")
+    assert_count(sep_each)
 
     fld <- get_idd_field(idd_env, class, property = c("units", "ip_units"), all = all)
 
